@@ -1,9 +1,9 @@
 package Text::Xslate::Syntax::Kolon;
-use Any::Moose;
+use Mouse;
 
 extends qw(Text::Xslate::Parser);
 
-no Any::Moose;
+no Mouse;
 __PACKAGE__->meta->make_immutable();
 
 __END__
@@ -202,7 +202,7 @@ via the dot-name syntax.
         : else {
             odd
         : }
-        : $i~.cycle("even", "odd") # => "even" -> "odd" -> "even" -> "odd" ...
+        : $~i.cycle("even", "odd") # => "even" -> "odd" -> "even" -> "odd" ...
     : }
 
 Supported iterator elements are C<index :Int>, C<count :Int>,
@@ -543,6 +543,10 @@ another example to use this block filter syntax.
 
 Note that closures are not supported.
 
+=head2 Chomping newlines
+
+You can add C<-> to the immediate start or end of a directive tag to control the newline chomping options to keep the output clean. The starting C<-> removes leading newlines and the ending C<-> removes trailing ones.
+
 =head2 Special keywords
 
 There are special keywords:
